function conn_batchexample_Preprocessing;
% example batch process for PREPROCESSING. Edit the fields below
% This example performs the following preprocessing steps:
% removal of confounding effects based on White matter / CSF CompCor, 3
% dimensions each, with additional removal of one task condition (named "rest"),
% and estimated subject motion parameters (named "motion"), and band-pass
% filtering

%% BATCH INFORMATION (edit this section)
rootfolder=pwd;
batch.filename=...                   % conn_* project file to be modified
    fullfile(rootfolder,'conn_attention.mat');

%% PREPROCESSING INFORMATION (edit this section to change the default values)
batch.Preprocessing.filter=[.01, inf];           % frequency filter (band-pass values, in Hz)
batch.Preprocessing.confounds.names=...          % Effects to be included as confounds (cell array of effect names, effect names can be first-level covariate names, condition names, or noise ROI names)
    {'Grey Matter','White Matter','CSF','sessions','Effect of Fix','Effect of Stat','Effect of Natt','Effect of Att'};
batch.Preprocessing.confounds.dimensions=...     % dimensionality of each effect listed above (cell array of values, leave empty a particular value to set to the default value -maximum dimensions of the corresponding effect-)
    {1, 5, 5, [], [],[],[],[]};
batch.Preprocessing.confounds.deriv=...          % derivatives order of each effect listed above (cell array of values, leave empty a particular value to set to the default value)
    {0, 0, 0, 1, 0, 0, 0, 0};

batch.Preprocessing.done=1;
batch.Preprocessing.overwrite='Yes';             % overwrite existing results if they exist (set to 'No' if you want to skip preprocessing steps for subjects/ROIs already analyzed)    

conn_batch(batch);

