% This example prompts the user to select one subject functional series
% (one resting state session) and one anatomical volume, and performs all 
% standard SPM preprocessing steps and connectivity analyses using the 
% default ROIs as sources on a series of overlapping segments (1 minute 
% sliding-window with 30 sec overlap by default).
%

%% batch preprocessing for single-subject single-session data 
FUNCTIONAL_FILE=cellstr(spm_select(Inf,'\.img$|\.nii$','Select functional volumes'));
if isempty(FUNCTIONAL_FILE),return;end
STRUCTURAL_FILE=cellstr(spm_select(1,'\.img$|\.nii$','Select structural volume'));
if isempty(STRUCTURAL_FILE),return;end
TR=inputdlg('Enter Repetition-Time (in seconds)','TR',1,{num2str('2')});TR=str2num(TR{1});
SW1=inputdlg('Enter Sliding window length (in seconds)','Sliding window length',1,{num2str('60')});SW1=str2num(SW1{1});
SW2=inputdlg('Enter Sliding window overlap (in seconds)','Sliding window overlap',1,{num2str('30')});SW2=str2num(SW2{1});
SW0=inputdlg('Enter Sliding window offset (in seconds)','Sliding window offset',1,{num2str('0')});SW0=str2num(SW0{1});
USEDEFAULTS=0; % set to "1" to perform analyses for all default ROIs (instead of only the four default-network ROIs)

spm('defaults','fmri');
spm_jobman('initcfg');
clear matlabbatch;
% Realign
matlabbatch{1}.spm.spatial.realign.estwrite.data{1}=FUNCTIONAL_FILE;
% Coregister
matlabbatch{2}.spm.spatial.coreg.estimate.ref=conn_prepend('mean',{FUNCTIONAL_FILE{1}});
matlabbatch{2}.spm.spatial.coreg.estimate.source=STRUCTURAL_FILE;
% Segment
matlabbatch{3}.spm.spatial.preproc.data=STRUCTURAL_FILE;
matlabbatch{3}.spm.spatial.preproc.output.GM=[1,1,1];
matlabbatch{3}.spm.spatial.preproc.output.WM=[1,1,1];
matlabbatch{3}.spm.spatial.preproc.output.CSF=[1,1,1];
% Normalize
matlabbatch{4}.spm.spatial.normalise.write.subj.matname=conn_prepend('',STRUCTURAL_FILE,'_seg_sn.mat');
matlabbatch{4}.spm.spatial.normalise.write.subj.resample=conn_prepend('r',FUNCTIONAL_FILE);
matlabbatch{4}.spm.spatial.normalise.write.roptions.vox=[3 3 3];
matlabbatch{5}.spm.spatial.normalise.write.subj.matname=conn_prepend('',STRUCTURAL_FILE,'_seg_sn.mat');
matlabbatch{5}.spm.spatial.normalise.write.subj.resample =conn_prepend('m',STRUCTURAL_FILE);
matlabbatch{5}.spm.spatial.normalise.write.roptions.vox=[3 3 3];
% Smooth
matlabbatch{6}.spm.spatial.smooth.data=conn_prepend('wr',FUNCTIONAL_FILE);
matlabbatch{6}.spm.spatial.smooth.fwhm=[8,8,8];
save('job.mat','matlabbatch');
spm_jobman('run',matlabbatch);

%% CONN Setup
clear batch;
batch.filename=fullfile(pwd,'conn_data.mat');
if ~isempty(dir(batch.filename)), 
    Ransw=questdlg('conn_data project already exists, Overwrite?','warning','Yes','No','No');
    if strcmp(Ransw,'Yes'), batch.isnew=1; else, batch.isnew=0; end
else,batch.isnew=1;end
batch.Setup.nsubjects=1;
batch.Setup.RT=TR;
batch.Setup.functionals{1}{1}=char(conn_prepend('swr',FUNCTIONAL_FILE));
batch.Setup.structurals{1}=char(conn_prepend('wm',STRUCTURAL_FILE));
batch.Setup.masks.Grey{1}=char(conn_prepend('mwc1',STRUCTURAL_FILE));
batch.Setup.masks.White{1}=char(conn_prepend('mwc2',STRUCTURAL_FILE));
batch.Setup.masks.CSF{1}=char(conn_prepend('mwc3',STRUCTURAL_FILE));
if ~USEDEFAULTS,
batch.Setup.rois={...%fullfile(fileparts(which('conn')),'rois','BA.img'), ...
	fullfile(fileparts(which('conn')),'rois','MPFC.tal'),...
	fullfile(fileparts(which('conn')),'rois','PCC.tal'),...
	fullfile(fileparts(which('conn')),'rois','LLP.tal'),...
	fullfile(fileparts(which('conn')),'rois','RLP.tal') };
end
batch.Setup.covariates.names={'realignment'};      
batch.Setup.covariates.files{1}{1}{1}=char(conn_prepend('rp_',{FUNCTIONAL_FILE{1}},'.txt'));
temp=spm_vol(FUNCTIONAL_FILE);nscans=length(temp);clear temp;
batch.Setup.conditions.names{1}='All';
batch.Setup.conditions.onsets{1}{1}{1}=0;
batch.Setup.conditions.durations{1}{1}{1}=inf;
for n1=1:floor((nscans*TR-SW0)/SW2),
    batch.Setup.conditions.names{1+n1}=['Block',num2str(n1)];
    batch.Setup.conditions.onsets{1+n1}{1}{1}=SW0+SW2*(n1-1);
    batch.Setup.conditions.durations{1+n1}{1}{1}=SW1;
end
batch.Setup.overwrite='Yes';             
batch.Setup.done=1;
%% CONN Preprocessing
batch.Preprocessing.filter=[0.01, 0.1];          % frequency filter (band-pass values, in Hz)
batch.Preprocessing.confounds.names=...          
    {'White Matter','CSF','realignment','Effect of All'};
batch.Preprocessing.confounds.dimensions=...     
    {5, 5, [], []};
batch.Preprocessing.confounds.deriv=...          
    {0, 0, 1, 0};
batch.Preprocessing.done=1;
batch.Preprocessing.overwrite='Yes';
%% CONN Analysis
batch.Analysis.analysis_number=1;
batch.Analysis.measure=1;               % connectivity measure used {1 = 'correlation (bivariate)', 2 = 'correlation (semipartial)', 3 = 'regression (bivariate)', 4 = 'regression (multivariate)';
batch.Analysis.weight=2;                % within-condition weight used {1 = 'none', 2 = 'hrf', 3 = 'hanning';
if ~USEDEFAULTS,
batch.Analysis.sources=...              % Sources names (seeds) for connectivity analyses - these correspond to a subset of ROI file names in the ROI folder (if this variable does not exist the toolbox will perform the analyses for all of the ROI files imported in the Setup step which are not defined as confounds in the Preprocessing step 
    {'MPFC','PCC'};
end
batch.Analysis.done=1;
batch.Analysis.overwrite='Yes';

conn_batch(batch);

%% CONN Display
conn
conn('load',fullfile(pwd,'conn_data.mat'));
conn gui_results


