function V=conn_create_vol(filename,V1,voxels,V0,sfilename,mask);

% CONN_CREATE_VOL creates CONN volume data from SPM volume and list of voxel indices
% conn_create_vol(fileoutput,spmVsource,voxels,spmVreference,sfilename_reference_to_source,mask);
%

if ischar(V1), V1=spm_vol(V1); end
if nargin<4 || isempty(V0), V0=V1; end
if nargin<5, sfilename=[]; end
if nargin<6, mask=[]; end
if ischar(V0), V0=spm_vol(V0); end
if isempty(voxels), voxels=1:prod(V0.dim(1:3)); 
else, voxels=sort(voxels); end
if ischar(mask), mask=spm_vol(mask); end
%if ~isempty(mask), maskX=spm_read_vols(mask); end

Nt=prod(size(V1));
voxels=voxels(:);
V.fname=filename;
V.matdim=struct('mat',V0(1).mat,'dim',V0(1).dim);
V.size=struct('Nt',Nt,'Ns',V0(1).dim(3),'Nv',zeros(1,V0(1).dim(3)));
V.voxels=[];
nslice=1+floor(voxels/prod(V0(1).dim(1:2)));
Nv=hist(nslice,1:V0(1).dim(3));

handle=fopen([V.fname,'c'],'wb');
fclose(handle);

V.GM=0;
N=0;
V.gm=zeros(Nt,1);
for slice=1:V0(1).dim(3),
	idx=sum(Nv(1:slice-1))+(1:Nv(slice));
	if ~isempty(idx),
        xyz=conn_convertcoordinates('idx2tal',voxels(idx),V0(1).mat,V0(1).dim);
        if ~isempty(sfilename),
            xyzs=conn_obj_coords2(sfilename, xyz')';
            if size(xyzs,2)<4, xyzs=cat(2,xyzs,ones(size(xyzs,1),1)); end
        else, xyzs=xyz; end
    
		if ~isempty(mask),
			z=spm_get_data(mask,pinv(mask(1).mat)*xyz');
			idx0=find(z>.5);
		else, idx0=1:length(idx); end
		if ~isempty(idx0),
			x=zeros(length(V1),length(idx0));
			for nscan=1:length(V1),
				x(nscan,:)=spm_get_data(V1(nscan),pinv(V1(nscan).mat)*xyzs(idx0,:)');
			end
			mx=mean(x,1);
			V.GM=V.GM+sum(mx);
			N=N+size(x,2);
			V.gm=V.gm+sum(x,2);
			for n1=1:size(x,2), x(:,n1)=x(:,n1)-mx(n1); end
			V.size.Nv(slice)=length(idx0);
			V.voxels=cat(1,V.voxels,voxels(idx(idx0)));
			conn_write_slice(V,x,slice);
		end
	end
end
V.GM=V.GM/N;
V.gm=V.gm/N;
V.scale=100/V.GM;
V.gm=V.gm*V.scale;
V.voxelsinv=zeros(V0(1).dim(1:3)); 
V.voxelsinv(V.voxels)=reshape(1:length(V.voxels),size(V.voxels));

if str2num(version('-release'))>=14, save(filename,'-V6','V');
else, save(filename,'V'); end
